#! /usr/bin/python3



__author__ = 'Predrag Pejovic'
__license__ = 'GPLv3'
__date__ = '2022-12-14'



from pylab import *
from oscusb3 import *
import usbtmc
import time
from engineeringnotation import *


################################################################################
#
# generator
#
gen = usbtmc.Instrument(0x0957, 0x0407)
gen.write('output off')
gen.write('function sinusoid')
gen.write('voltage:amplitude 0.020')
gen.write('voltage:offset 0')
gen.write('frequency 50')
gen.write('output:load 9.9e37')
#
def genamp(amplitude):
    gen.write('voltage:amplitude ' + str(amplitude))
#
def genfreq(frequency):    
    gen.write('frequency ' + str(frequency))
#
def waigen():
    gr = '0'
    while gr != '1':
        gr = gen.ask('*opc?')
#
################################################################################



################################################################################
#
o = Oscilloscope(shy = True)
#
o.write('select:ch1 off')
#
o.write('select:ch2 on')
o.write('ch2:position 0')
o.write('ch2:probe 1')
o.write('ch2:bandwidth on')
o.write('ch2:scale 0.2')
#
o.write('horizontal:main:scale 5e-3')
o.write('horizontal:position 0')
#
o.write('trigger:main:mode auto; type edge; slope rise')
o.write('trigger:main:edge:source ext')
o.write('trigger:main:level 0.7')
#
o.write('measurement:meas1:source ch2; type cmean')
#
o.waituntilready()
#
################################################################################



t1 = time.time()

print()

amplitudes = arange(0.0, 20.5, 0.5)
amplitudes[0] += 0.020

scales = array([0.002, 0.005, 0.010, 0.020, 0.050, 0.1, 0.2, 0.5, 1.0, 2.0, 5.0])
spanes = 4.0 * scales

names = ['sinusoid', 'square', 'ramp']
dnames = ['sinusoid:', 'square:  ', 'ramp:    ']
outputs = {}

genamp(amplitudes[0])
gen.write('output on')
time.sleep(1)

o.write('measurement:immed:type cmean; source ch2')

for i in range(3):
    name = names[i]
    gen.write('function ' + name)
    outputs[name] = []
    for a in amplitudes:
        genamp(a)
        waigen()
        scale = scales[np.where(a / 2.0 < spanes)[0][0]]
        o.write('ch2:scale ' + str(scale))
        print(dnames[i] + ' Vm = ' + toengstring(a / 2.0, fmtm = '6.2f') + \
        'V; scale = ' + toengstring(scale, fmtm = '6.2f') + 'V/div')
        time.sleep(1)
        #o.getpng(name + '_{:05.2f}'.format(a / 2.0))
        outputs[name].append(o.getvalue())

#np.save('trace-1-amplitudes.npy', amplitudes)
#np.save('trace-1-sin.npy', array(outputs[names[0]]))
#np.save('trace-1-square.npy', array(outputs[names[1]]))
#np.save('trace-1-ramp.npy', array(outputs[names[2]]))

close('all')

figure(1)
plot(amplitudes / 2.0, outputs[names[0]], 'b', label = names[0])
plot(amplitudes / 2.0, outputs[names[0]], 'b.')
plot(amplitudes / 2.0, outputs[names[1]], 'r', label = names[1])
plot(amplitudes / 2.0, outputs[names[1]], 'r.')
plot(amplitudes / 2.0, outputs[names[2]], 'g', label = names[2])
plot(amplitudes / 2.0, outputs[names[2]], 'g.')
xlabel('Vinm [V]')
ylabel('Vout [V]')
legend(loc = 'upper left')
#savefig('traces-1.pdf', bbox_inches = 'tight')
savefig('traces-1.png', bbox_inches = 'tight')

gen.write('function sinusoid')
gen.write('output off')

o.write('measurement:meas1:source ch2; type none')



t2 = time.time()
deltat = t2 - t1
m = floor(deltat / 60)
s = deltat - 60 * m

print()
print('completed in {:d} minutes {:.2f} seconds'.format(int(m), s))
print()

