#! /usr/bin/python3



__author__ = 'Predrag Pejovic'
__license__ = 'GPLv3'
__date__ = '2022-12-14'



from pylab import *
from oscusb3 import *
import usbtmc
import time



################################################################################
#
# generator
#
gen = usbtmc.Instrument(0x0957, 0x0407)
gen.write('output off')
gen.write('function sinusoid')
gen.write('voltage:offset 0')
gen.write('output:load 9.9e37')
#
def genamp(amplitude):
    gen.write('voltage:amplitude ' + str(amplitude))
#
def genfreq(frequency):    
    gen.write('frequency ' + str(frequency))
#
def waigen():
    gr = '0'
    while gr != '1':
        gr = gen.ask('*opc?')
#
gen.write('frequency 10')
genamp(10)
gen.write('output on')
#
################################################################################



################################################################################
#
o = Oscilloscope(shy = True)
#
o.write('select:ch1 on')
o.write('ch1:position 0')
o.write('ch1:probe 1')
o.write('ch1:bandwidth on')
o.write('ch1:scale 2')
#
o.write('select:ch2 on')
o.write('ch2:position 0')
o.write('ch2:probe 1')
o.write('ch2:bandwidth on')
o.write('ch2:scale 2')
#
o.write('horizontal:scale 10e-3')
o.write('horizontal:position 0')
#
o.write('trigger:main:edge:source ext')
o.write('trigger:main:level 0.7')
#
o.waituntilready()
#
################################################################################



t1 = time.time()

f = logspace(1, 6, 51)
n = len(f)
v1 = empty(n)
v2 = empty(n)

wholef = array([
2.0, 4.0, 10.0, 20.0, 40.0, 100.0, 200.0, 400.0, 1e3, 2e3, 4e3, 10e3, 
20e3, 40e3, 100e3, 200e3, 400e3, 1e6, 2e6, 4e6, 10e6, 20e6
])

timescales = 0.1 / wholef

twait = 0.5

print()



for i in range(n):

    print('{:6.2f} %'.format(float(i) / n * 100))

    assert 2.0 <= f[i] and 20e6 >= f[i]

    genfreq(f[i])
    
    findex = where(wholef <= f[i])[0][-1]
    ratio = wholef[findex] / f[i]
    ns = int(round(2500 * ratio))

    o.write('horizontal:scale ' + str(timescales[findex]))
    o.waituntilready()

    time.sleep(twait)

    o.stop()
    v1s = o.getsamples(1, start = 1, stop = ns)
    v2s = o.getsamples(2, start = 1, stop = ns)
    o.run()

    v1[i] = mean(v1s)
    v2[i] = mean(v2s)

gen.write('output off')

o.write('ch1:bandwidth off')
o.write('ch2:bandwidth off')



close('all')
#
figure(1)
plot(f / 1000.0, v1, 'b', label = 'v_1')
plot(f / 1000.0, v1, 'b.')
plot(f / 1000.0, v2, 'g', label = 'v_2')
plot(f / 1000.0, v2, 'g.')
xlim(-10, 1010)
xlabel('f [kHz]')
ylabel('v_1, v_2 [V]')
title('peak detector')
legend(loc = 'center right')
savefig('fresponse.png', bbox_inches = 'tight')
#savefig('fresponse.pdf', bbox_inches = 'tight')
#
figure(2)
loglog(f, (v1 - v2) / 10, 'b')
loglog(f, (v1 - v2) / 10, 'b.')
xlabel('f [Hz]')
ylabel('A')
title('peak detector')
savefig('fresponseloglog.png', bbox_inches = 'tight')
#savefig('fresponseloglog.pdf', bbox_inches = 'tight')



t2 = time.time()
deltat = t2 - t1
m = floor(deltat / 60)
s = deltat - 60 * m

print()
print('completed in {:d} minutes {:.2f} seconds'.format(int(m), s))
print()

